
/*
 *   XSTREAM.C   - execute a program on each member of input stream.
 *
 *                            Copywrite 1988: Mark Pharis
 *                                            170 Rebecca Dr.
 *                                            Hendersonville, TN 37075
 *                                            CIS 76244,452
 * 
 *     COMPILE: cl xstream.c \path\setargv.obj /link /NOE
 *
 *     This program receives a list of filespecs either by way of expanding
 *     the wild card parameters given by the user on the command line, or
 *     by way of a pipe from another program.  It then executes whatever 
 *     program the user specified on the command line, using the stream of
 *     file specifications as parameters to the executable program.
 *
 *     This is a skelatal program.  The only validity test is for parameter
 *     usage, and no attempt is made to "clean up" if interrupted.  An
 *     immediate improvement to this program would be using signal() to
 *     sense interrupts and, say, put us back in the original directory, if
 *     we've been moved.
 *          
 *     The use of the function, spawnlp(), allows us to list the parameters 
 *     rather than build an *ptrs[], however, usage of spawnle() or spawnlpe()
 *     could gain the ability to control the environment passed to the user's
 *     program.
 *
 *     Examples:
 *       1) xstream list *.c
 * 
 *          This example executes the popular program, LIST, against each
 *          C source file in the current directory.
 *
 *       2) fdf c:\*.asm -s|xstream list
 *
 *          This example of XSTREAM receives the list of filespecs from
 *          the output of FDF (File Directory Filter).  XSTREAM will
 *          run the program, LIST, against each file received in the 
 *          piped input stream.
 *          In this example, the program FDF has been told to produce a
 *          list of each *.asm files, starting at the root directory of
 *          drive c:, and recursively going thru all the subdirectories 
 *          under root.  The output of FDF, which becomes the input to
 *          XSTREAM, might look like this:
 *              C:\CFILES\XSTREAM.C
 *              C:\CFILES\FDF.C
 *              C:\CFILES\SINCE.C
 *              C:\LIB\SETARGV.C
 *              C:\LIB\C0M.C
 *
 *
 *     As might be noticed, the more useful application of XSTREAM is in
 *     its processing of piped input.  Not all programs write their output
 *     to stdout in a fashion suitable for pipes.  The basic rules are 
 *     simple: 
 *       1) Keep the output to Standard Out (stdout) clean and simple so
 *          another program can handle it.
 *       2) Write to Standard Error (stderr) all the other stuff, like error
 *          messages, banners, copywrites, monetary solicitations, etc.
 *       3) Use the function, isatty(), in programs to find out whether the
 *          standard input is connected to the console or another program.
 *
 *     A few programs that try to take advantage of this "open architecture"
 *     are:   FDF    - a file directory filter by this author
 *            SINCE  - a date filter by this author
 *            FGREP  - a Regular Expression filter.  Designed at Bell Labs,
 *                     latest and greatest DOS version I've seen by Chris
 *                     Dunford
 *            FDUMP  - hex and ASCII dump/search by this author
 *            SORT, MORE - MicroSoft  
 *__________________________________________________________________________*/

#include <stdio.h>
#include <process.h>

main(ac,av)
int  ac;
char *av[];
{
	char	input[80];
	int		i =0;	

	if( !strcmp(av[1],"-h")) 
		help();
	

	else	if( isatty(0) && ac >2)
				while( *av[2+i]) {
		    		spawnlp( P_WAIT,av[1],av[1],av[2+i],NULL);
		    		++i;
		     		}


	else	if( !isatty(0) && ac ==2)
				while(gets(input))
					spawnlp( P_WAIT,av[1],av[1],input,NULL);


	else {
		fprintf( stderr,"\n%s: invalid number of arguments.", av[0]);
		fprintf( stderr,"\nFor HELP, enter 'xstream -h'.\n");
		exit(-1);
		}


	exit(0);
}


/*---------------------------------------------------------------
|   help()   - displays help screen for user.
|            - called only when user specifies "xstream -h".
|            - output is to stdout.
|_______________________________________________________________*/

help() {
puts("\n                            XSTREAM   1.0");
puts("             a multiple-file executor that handles pipes");
puts("");
puts("                  Mark Pharis, HendSoft, July 7,1988");
puts("");
puts("");
puts("");
puts("Usage examples:");
puts("         1)   xstream program filespec");
puts("         xstream LIST *.c");
puts("         xstream TOUCH *.*");     
puts("         2)   program1 | xstream program2");
puts("         FDF \pdev\*.asm -s | xstream LIST");
puts("         SINCE 05/06 *.c | xstream LIST");
puts("         FDF c:\*.* -H |SINCE 5/6 *.* | xstream LIST");
puts("");
puts("See the file, XSTREAM.C for more complete information.");
}

/****************************  EOF  ***************************/